from edc_adherence.model_mixins import MedicationAdherenceModelMixin
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin


class DmMedicationAdherence(
    MedicationAdherenceModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    condition_label = "Diabetes"

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Medication Adherence (Diabetes)"
        verbose_name_plural = "Medication Adherence (Diabetes)"



from django.db import models
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_dx_review.model_mixins import rx_initial_review_model_mixin_factory
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin, FollowupReviewModelMixin


class HivReview(
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="antiretroviral therapy (ART)"
    ),
    FollowupReviewModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    dx = models.CharField(
        verbose_name="Has the patient been infected with HIV?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    # arv_initiated = models.CharField(
    #     verbose_name="Has the patient started antiretroviral therapy (ART)?",
    #     max_length=15,
    #     choices=YES_NO_NA,
    #     default=NOT_APPLICABLE,
    #     help_text="Select `not applicable` if previously reported.",
    # )
    # arv_initiation_actual_date = models.DateField(
    #     verbose_name="Date started antiretroviral therapy (ART)",
    #     null=True,
    #     blank=True,
    # )

    @property
    def best_art_initiation_date(self):
        return self.rx_init_date or self.rx_init_calculated_date

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "HIV Review"
        verbose_name_plural = "HIV Review"



from __future__ import annotations

from dateutil.relativedelta import relativedelta
from edc_visit_schedule import Visit as BaseVisit
from edc_visit_schedule.constants import MONTH0, MONTH12

from .crfs import crfs_d1, crfs_followup, crfs_missed
from .crfs import crfs_prn as default_crfs_prn
from .crfs import crfs_unscheduled as default_crfs_unscheduled
from .requisitions import requisitions_d1, requisitions_followup
from .requisitions import requisitions_prn as default_requisitions_prn
from .requisitions import requisitions_unscheduled as default_requisitions_unscheduled


def get_visit_code(mnth: int):
    if mnth < 10:
        return f"10{mnth*10}"
    return f"1{mnth*10}"


def get_followup_visit(month):
    if month == 0 or month >= 12:
        raise ValueError(f"Invalid month number. Follow visits are from 1-11. Got {month}.")
    rlower = relativedelta(days=13)
    rupper = relativedelta(days=15)
    return Visit(
        code=get_visit_code(month),
        title=f"Followup month {month}",
        timepoint=month,
        rbase=relativedelta(months=month),
        rlower=rlower,
        rupper=rupper,
        add_window_gap_to_lower=True,
        requisitions=requisitions_followup,
        crfs=crfs_followup,
        crfs_unscheduled=crfs_followup,
        facility_name="5-day-clinic",
    )


class Visit(BaseVisit):
    def __init__(
        self,
        crfs_unscheduled=None,
        requisitions_unscheduled=None,
        crfs_prn=None,
        requisitions_prn=None,
        allow_unscheduled=None,
        **kwargs,
    ):
        super().__init__(
            allow_unscheduled=True if allow_unscheduled is None else allow_unscheduled,
            crfs_unscheduled=crfs_unscheduled or default_crfs_unscheduled,
            requisitions_unscheduled=requisitions_unscheduled
            or default_requisitions_unscheduled,
            crfs_prn=crfs_prn or default_crfs_prn,
            requisitions_prn=requisitions_prn or default_requisitions_prn,
            crfs_missed=crfs_missed,
            **kwargs,
        )


visit00 = Visit(
    code=MONTH0,
    title="Baseline",
    timepoint=0,
    rbase=relativedelta(day=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=0),
    add_window_gap_to_lower=True,
    requisitions=requisitions_d1,
    crfs=crfs_d1,
    facility_name="5-day-clinic",
)


visit12 = Visit(
    code=MONTH12,
    title="Month 12 (End of study)",
    timepoint=12,
    rbase=relativedelta(months=12),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    add_window_gap_to_lower=True,
    requisitions=requisitions_followup,
    crfs=crfs_followup,
    facility_name="5-day-clinic",
)
